/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.ws.mdp;

import java.util.LinkedList;
import java.util.List;

/**
 *
 * @author Karel Horak
 */
public class MDPState {

    private String name;
    private List<MDPAction> actions = new LinkedList<MDPAction>();
    private double evaluation = 0;
    private double newEvaluation = 0;

    public MDPState(String name) {
        this.name = name;
    }

    public void addAction(MDPAction a) {
        actions.add(a);
    }

    public void setEvaluation(double evaluation) {
        this.evaluation = evaluation;
    }

    public double getEvaluation() {
        return evaluation;
    }

    public void update(double discount) {
        newEvaluation = Double.NEGATIVE_INFINITY;
        if (actions.isEmpty()) {
            newEvaluation = evaluation;
            return;
        }
        for (MDPAction a : actions) {
            //System.out.printf("For action %s\n", a.getName());
            double eval = 0.0;
            for (MDPOutcome t : a.getTransitions()) {
                //System.out.printf("Questioning %s - p=%1.4f e=%3.4f\n", t.getState().getName(), t.getProbability(), t.getState().getEvaluation());
                eval += t.getProbability() * t.getState().getEvaluation();
            }

            if (eval > newEvaluation) {
                newEvaluation = eval;
            }
        }

        newEvaluation -= discount;
    }

    public boolean commit(double threshold) {
        boolean impChange = Math.abs(newEvaluation - evaluation) > threshold;
        evaluation = newEvaluation;

        return impChange;
    }

    public String getName() {
        return name;
    }

    public MDPAction getOptimalAction() {
        double outcome = Double.NEGATIVE_INFINITY;
        MDPAction optimal = null;
        for (MDPAction a : actions) {
            double cOutcome = 0;
            for (MDPOutcome t : a.getTransitions()) {
                cOutcome += t.getProbability() * t.getState().getEvaluation();
            }

            if (cOutcome > outcome) {
                outcome = cOutcome;
                optimal = a;
            }
        }
        
        return optimal;
    }
}
